/* ojnk */

#include <stdio.h>
#include <unistd.h>
#include <errno.h>
#include <netdb.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/syslog.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#define HIGH_UID 32000
#define HIGH_GID 20000

#define FACILITY LOG_AUTHPRIV
#define SEVERITY LOG_ALERT

#define MESSAGE (strerror(EACCES))

static char *host_resolve(struct in_addr addr) {
	struct hostent *host = NULL;

	host = gethostbyaddr((char *) &addr, sizeof(struct in_addr), AF_INET);

	if (host != NULL)
		return (host->h_name);
	else
		return (inet_ntoa(addr));

	return ("unknown");		
}

void check_environment(const char *whoami) {
	struct sockaddr_in sin;
	size_t len = sizeof(sin);
	uid_t uid, euid;
	gid_t gid, egid;

	openlog(whoami, LOG_CONS | LOG_PID | LOG_NDELAY, FACILITY);

	if (!getpeername(fileno(stdin), (struct sockaddr *) &sin, &len)) {
		if (sin.sin_family == AF_INET) {
			syslog(SEVERITY, "Remote execution attempt from %s",
					host_resolve(sin.sin_addr));
        	exit(-1);
		}
	}

	uid = getuid();
	euid = geteuid();

	if (uid != euid) {
		syslog(SEVERITY, "UID [%u] does not match EUID [%u]", uid, euid);
		puts(MESSAGE);
		exit(-1);
	}

	if (uid > HIGH_UID) {
		syslog(SEVERITY, "UID [%u] is higher than the %u limit", uid, HIGH_UID);
		puts(MESSAGE);
		exit(-1);
	}

	gid = getgid();
	egid = getegid();

	if (gid != egid) {
		syslog(SEVERITY, "GID [%u] does not match EGID [%u]", gid, egid);
		puts(MESSAGE);
		exit(-1);
	}

	if (gid > HIGH_GID) {
		syslog(SEVERITY, "GID [%u] is higher than the %u limit", gid, HIGH_GID);
		puts(MESSAGE);
		exit(-1);
	}

	closelog();
}
